uniform sampler2D	dirtMap;
				
uniform lowp float	dirtLightInfluence;				
uniform lowp float	dirtAngleNormalConfines;
uniform lowp float	dirtBlendingFactor;
uniform lowp float	dirtAmount;
uniform lowp float	dirtScale;
uniform lowp float	dirtCoefficient;
uniform lowp vec3	minColorTo;
uniform lowp vec3	maxColorTo;
					
uniform highp	vec3	fxRootZeroPos;
uniform lowp	mat4	inRotationMatrix;
					
uniform lowp vec4 dirtUVXZ;
uniform lowp vec4 dirtUVYZ;
uniform lowp vec2 dirtFrontUVOffset;
uniform lowp vec2 dirtBackUVOffset;
uniform lowp vec2 dirtSidesUVOffset;				
uniform lowp vec2 carFrontSizeScale;
uniform lowp vec2 carBackSizeScale;
uniform lowp vec2 carSidesSizeScale;		

lowp vec2 rotate(lowp vec2 center, lowp vec2 point, lowp float angle)
{
	lowp float s = sin(angle);
	lowp float c = cos(angle);

	// translate point back to origin:
	point.x -= center.x;
	point.y -= center.y;

	// rotate point
	lowp float xnew = point.x * c - point.y * s;
	lowp float ynew = point.x * s + point.y * c;

	// translate point back:
	point.x = xnew + center.x;
	point.y = ynew + center.y;
					  
	return point;
}

lowp float rad(lowp float degree)
 {
	return degree * 3.14 / 180.0;
 }

lowp vec2 CutUV(lowp vec2 uv, lowp vec2 start, lowp vec2 end)
{
	return vec2(
		start.x + (end.x - start.x) * uv.x,
		start.y + (end.y - start.y) * uv.y
	);
}

lowp vec2 CutUV(lowp vec2 uv, lowp vec4 restriction)
{
	return vec2(
		restriction.x + (restriction.z - restriction.x) * uv.x,
		restriction.y + (restriction.w - restriction.y) * uv.y
	);
}


lowp vec2 GetSphericUV(lowp vec3 pos)
{
	lowp float angle_radius = acos( dot(normalize(pos), vec3(0.0, 0.0, 1.0)) );
	lowp float radius = angle_radius / 3.14;
	lowp float angle_uv = acos(dot(normalize(pos.xy), vec2(0.0, 1.0)) ) * (pos.x/abs(pos.x));
	return rotate(vec2(0.5, 0.5), vec2( 0.5, 0.5 + radius), angle_uv);
}
					
lowp vec2 GetCircularUV(lowp vec3 pos)
{
	lowp float angle_uv = (acos(dot(normalize(pos.xy), vec2(0.0, 1.0)) ) * (pos.x/abs(pos.x)) + 3.14) / (3.14 * 2.0);
	return vec2(angle_uv, 1.0 - pos.z);
}
					
lowp vec2 GetPlanarUV(lowp vec2 pos, lowp vec2 scale)
{
	return vec2( pos.x * scale.x + 0.5, (1.0 - pos.y) * scale.y);;
}

lowp vec4 GetSphericDirt(sampler2D tex, lowp vec3 pos)
{
	return texture2D( tex, GetSphericUV( pos ) );
}

lowp vec4 GetCircularDirt(sampler2D tex, lowp vec3 pos)
{
	return texture2D( tex, GetCircularUV( pos ) );
}

lowp vec3 MixPlanarDirt(lowp vec3 color, lowp vec3 light)
{
	if(dirtCoefficient > 0.0)
	{
		highp vec3 blending = abs( vNormal );
		blending = normalize(max(blending, 0.00001)); // Force weights to sum to 1.0
		highp float b = (blending.x + blending.y + blending.z);
		blending /= vec3(b, b, b);
							
#ifdef SHADER_IN_LED
		// LED not translate rotate matrix, so for preview using not rotated objects
		lowp vec3 local_pos = vPosWorld.xyz - fxRootZeroPos;
		lowp vec3 normal = normalize(vNormal);
#else
		lowp vec3 local_pos = (vec4(vPosWorld.xyz - fxRootZeroPos, 1.0) * inRotationMatrix).xyz;
		lowp vec3 normal = (vec4(normalize(vNormal), 1.0) * inRotationMatrix).xyz;
#endif
		// XZ front-rear
		// YZ sides
		lowp float front = step( 0.0, -local_pos.y);
		lowp vec2 dirtXZUV = CutUV( GetPlanarUV(local_pos.xz, mix(carBackSizeScale, carFrontSizeScale, front) ), dirtUVXZ ) + mix(dirtBackUVOffset, dirtFrontUVOffset, front);
		lowp vec2 dirtYZUV = CutUV( GetPlanarUV(local_pos.yz, carSidesSizeScale), dirtUVYZ ) + dirtSidesUVOffset;
		
		lowp float a = rad(dirtAngleNormalConfines);
		lowp float normal_angle_confines_xz = step( a, abs(abs( acos( dot(normalize(normal), vec3(0.0, 1.0, 0.0)) ) ) - 1.57)) ;
		lowp float normal_angle_confines_yz = step( a, abs(abs( acos( dot(normalize(normal), vec3(1.0, 0.0, 0.0)) ) ) - 1.57)) ;
							
		lowp vec4 DIRT = texture2D(dirtMap, dirtXZUV) * normal_angle_confines_xz + texture2D(dirtMap, dirtYZUV) * normal_angle_confines_yz;
							
		DIRT = mix(DIRT, DIRT * blending.x, dirtBlendingFactor);
		DIRT.a = DIRT.r * dirtCoefficient;
		DIRT.rgb = mix(minColorTo, maxColorTo, DIRT.b);						
#ifdef SHADER_IN_LED
		// LED not translate dynamic 'dirtCoefficient', for good visual using full dirt
		DIRT.rgb = mix(minColorTo, maxColorTo, DIRT.a);
#endif
		lowp float dirtFactor = min(dirtAmount * DIRT.a, 1.0); // compute the dirtFactor
		
#ifdef USE_CAR_QUALITY_LIGHTING
		DIRT.rgb = mix(DIRT.rgb, DIRT.rgb * light, dirtLightInfluence); // modulate the dirt with the light
#endif
		return mix(color, DIRT.rgb, dirtFactor);
	}
	
	//default
	return color;
}